import * as plugins from '../../../plugins.js';
import type { IAcmeOptions } from '../../../certificate/models/certificate-types.js';
import type { TForwardingType } from '../../../forwarding/config/forwarding-types.js';

/**
 * Supported action types for route configurations
 */
export type TRouteActionType = 'forward' | 'redirect' | 'block' | 'static';

/**
 * TLS handling modes for route configurations
 */
export type TTlsMode = 'passthrough' | 'terminate' | 'terminate-and-reencrypt';

/**
 * Port range specification format
 */
export type TPortRange = number | number[] | Array<{ from: number; to: number }>;

/**
 * Route match criteria for incoming requests
 */
export interface IRouteMatch {
  // Listen on these ports (required)
  ports: TPortRange;

  // Optional domain patterns to match (default: all domains)
  domains?: string | string[];

  // Advanced matching criteria
  path?: string;           // Match specific paths
  clientIp?: string[];     // Match specific client IPs
  tlsVersion?: string[];   // Match specific TLS versions
  headers?: Record<string, string | RegExp>; // Match specific HTTP headers
}

/**
 * Context provided to port and host mapping functions
 */
export interface IRouteContext {
  // Connection information
  port: number;          // The matched incoming port
  domain?: string;       // The domain from SNI or Host header
  clientIp: string;      // The client's IP address
  serverIp: string;      // The server's IP address
  path?: string;         // URL path (for HTTP connections)
  query?: string;        // Query string (for HTTP connections)
  headers?: Record<string, string>; // HTTP headers (for HTTP connections)

  // TLS information
  isTls: boolean;        // Whether the connection is TLS
  tlsVersion?: string;   // TLS version if applicable

  // Route information
  routeName?: string;    // The name of the matched route
  routeId?: string;      // The ID of the matched route

  // Target information (resolved from dynamic mapping)
  targetHost?: string;   // The resolved target host
  targetPort?: number;   // The resolved target port

  // Additional properties
  timestamp: number;     // The request timestamp
  connectionId: string;  // Unique connection identifier
}

/**
 * Target configuration for forwarding
 */
export interface IRouteTarget {
  host: string | string[] | ((context: any) => string | string[]);  // Support static or dynamic host selection with any compatible context
  port: number | ((context: any) => number);  // Support static or dynamic port mapping with any compatible context
  preservePort?: boolean;   // Use incoming port as target port (ignored if port is a function)
}

/**
 * TLS configuration for route actions
 */
export interface IRouteTls {
  mode: TTlsMode;
  certificate?: 'auto' | {   // Auto = use ACME
    key: string;
    cert: string;
  };
}

/**
 * Redirect configuration for route actions
 */
export interface IRouteRedirect {
  to: string;            // URL or template with {domain}, {port}, etc.
  status: 301 | 302 | 307 | 308;
}

/**
 * Authentication options
 */
export interface IRouteAuthentication {
  type: 'basic' | 'digest' | 'oauth' | 'jwt';
  credentials?: {
    username: string;
    password: string;
  }[];
  realm?: string;
  jwtSecret?: string;
  jwtIssuer?: string;
  oauthProvider?: string;
  oauthClientId?: string;
  oauthClientSecret?: string;
  oauthRedirectUri?: string;
  [key: string]: any;  // Allow additional auth-specific options
}

/**
 * Security options for route actions
 */
export interface IRouteSecurity {
  allowedIps?: string[];
  blockedIps?: string[];
  maxConnections?: number;
  authentication?: IRouteAuthentication;
}

/**
 * Static file server configuration
 */
export interface IRouteStaticFiles {
  root: string;
  index?: string[];
  headers?: Record<string, string>;
  directory?: string;
  indexFiles?: string[];
  cacheControl?: string;
  expires?: number;
  followSymlinks?: boolean;
  disableDirectoryListing?: boolean;
}

/**
 * Test route response configuration
 */
export interface IRouteTestResponse {
  status: number;
  headers: Record<string, string>;
  body: string;
}

/**
 * URL rewriting configuration
 */
export interface IRouteUrlRewrite {
  pattern: string;            // RegExp pattern to match in URL
  target: string;             // Replacement pattern (supports template variables like {domain})
  flags?: string;             // RegExp flags like 'g' for global replacement
  onlyRewritePath?: boolean;  // Only apply to path, not query string
}

/**
 * Advanced options for route actions
 */
export interface IRouteAdvanced {
  timeout?: number;
  headers?: Record<string, string>;
  keepAlive?: boolean;
  staticFiles?: IRouteStaticFiles;
  testResponse?: IRouteTestResponse;
  urlRewrite?: IRouteUrlRewrite; // URL rewriting configuration
  // Additional advanced options would go here
}

/**
 * WebSocket configuration
 */
export interface IRouteWebSocket {
  enabled: boolean;                   // Whether WebSockets are enabled for this route
  pingInterval?: number;              // Interval for sending ping frames (ms)
  pingTimeout?: number;               // Timeout for pong response (ms)
  maxPayloadSize?: number;            // Maximum message size in bytes
  customHeaders?: Record<string, string>; // Custom headers for WebSocket handshake
  subprotocols?: string[];            // Supported subprotocols
  rewritePath?: string;               // Path rewriting for WebSocket connections
  allowedOrigins?: string[];          // Allowed origins for WebSocket connections
  authenticateRequest?: boolean;      // Whether to apply route security to WebSocket connections
}

/**
 * Load balancing configuration
 */
export interface IRouteLoadBalancing {
  algorithm: 'round-robin' | 'least-connections' | 'ip-hash';
  healthCheck?: {
    path: string;
    interval: number;
    timeout: number;
    unhealthyThreshold: number;
    healthyThreshold: number;
  };
}

/**
 * Action configuration for route handling
 */
export interface IRouteAction {
  // Basic routing
  type: TRouteActionType;

  // Target for forwarding
  target?: IRouteTarget;

  // TLS handling
  tls?: IRouteTls;

  // For redirects
  redirect?: IRouteRedirect;

  // For static files
  static?: IRouteStaticFiles;

  // WebSocket support
  websocket?: IRouteWebSocket;

  // Load balancing options
  loadBalancing?: IRouteLoadBalancing;

  // Security options
  security?: IRouteSecurity;

  // Advanced options
  advanced?: IRouteAdvanced;
  
  // Additional options for backend-specific settings
  options?: {
    backendProtocol?: 'http1' | 'http2';
    [key: string]: any;
  };
}

/**
 * Rate limiting configuration
 */
export interface IRouteRateLimit {
  enabled: boolean;
  maxRequests: number;
  window: number; // Time window in seconds
  keyBy?: 'ip' | 'path' | 'header';
  headerName?: string;
  errorMessage?: string;
}

/**
 * Security features for routes
 */
export interface IRouteSecurity {
  rateLimit?: IRouteRateLimit;
  basicAuth?: {
    enabled: boolean;
    users: Array<{ username: string; password: string }>;
    realm?: string;
    excludePaths?: string[];
  };
  jwtAuth?: {
    enabled: boolean;
    secret: string;
    algorithm?: string;
    issuer?: string;
    audience?: string;
    expiresIn?: number;
    excludePaths?: string[];
  };
  ipAllowList?: string[];
  ipBlockList?: string[];
}

/**
 * CORS configuration for a route
 */
export interface IRouteCors {
  enabled: boolean;                     // Whether CORS is enabled for this route
  allowOrigin?: string | string[];      // Allowed origins (*,domain.com,[domain1,domain2])
  allowMethods?: string;                // Allowed methods (GET,POST,etc.)
  allowHeaders?: string;                // Allowed headers
  allowCredentials?: boolean;           // Whether to allow credentials
  exposeHeaders?: string;               // Headers to expose to the client
  maxAge?: number;                      // Preflight cache duration in seconds
  preflight?: boolean;                  // Whether to respond to preflight requests
}

/**
 * Headers configuration
 */
export interface IRouteHeaders {
  request?: Record<string, string>;     // Headers to add/modify for requests to backend
  response?: Record<string, string>;    // Headers to add/modify for responses to client
  cors?: IRouteCors;                    // CORS configuration
}

/**
 * The core unified configuration interface
 */
export interface IRouteConfig {
  // Unique identifier
  id?: string;

  // What to match
  match: IRouteMatch;

  // What to do with matched traffic
  action: IRouteAction;

  // Custom headers
  headers?: IRouteHeaders;

  // Security features
  security?: IRouteSecurity;

  // Optional metadata
  name?: string;             // Human-readable name for this route
  description?: string;      // Description of the route's purpose
  priority?: number;         // Controls matching order (higher = matched first)
  tags?: string[];           // Arbitrary tags for categorization
  enabled?: boolean;         // Whether the route is active (default: true)
}

/**
 * Unified SmartProxy options with routes-based configuration
 */
export interface IRoutedSmartProxyOptions {
  // The unified configuration array (required)
  routes: IRouteConfig[];

  // Global/default settings
  defaults?: {
    target?: {
      host: string;
      port: number;
    };
    security?: IRouteSecurity;
    tls?: IRouteTls;
    // ...other defaults
  };

  // Other global settings remain (acme, etc.)
  acme?: IAcmeOptions;

  // Connection timeouts and other global settings
  initialDataTimeout?: number;
  socketTimeout?: number;
  inactivityCheckInterval?: number;
  maxConnectionLifetime?: number;
  inactivityTimeout?: number;
  gracefulShutdownTimeout?: number;

  // Socket optimization settings
  noDelay?: boolean;
  keepAlive?: boolean;
  keepAliveInitialDelay?: number;
  maxPendingDataSize?: number;

  // Enhanced features
  disableInactivityCheck?: boolean;
  enableKeepAliveProbes?: boolean;
  enableDetailedLogging?: boolean;
  enableTlsDebugLogging?: boolean;
  enableRandomizedTimeouts?: boolean;
  allowSessionTicket?: boolean;

  // Rate limiting and security
  maxConnectionsPerIP?: number;
  connectionRateLimitPerMinute?: number;

  // Enhanced keep-alive settings
  keepAliveTreatment?: 'standard' | 'extended' | 'immortal';
  keepAliveInactivityMultiplier?: number;
  extendedKeepAliveLifetime?: number;

  /**
   * Optional certificate provider callback. Return 'http01' to use HTTP-01 challenges,
   * or a static certificate object for immediate provisioning.
   */
  certProvisionFunction?: (domain: string) => Promise<any>;
}